local core = require("core")
local gmui = require("gmui")
local ui = require("UI/UI")
local shop = require("lazy_shop_lite")
local data = require("GameData/GameData")

local game_shop = {}
local game
local meta
local menu
local endless
local live_event

local function UpdateStoreItem_Pack(self)
    local product_id = self.item_data.product_id
    local stock_remaining = shop.GetItemStockAvailable(product_id)
    local item_available = stock_remaining > 0
    DCEI.SetFrameActive(self.PurchasedFrame, not item_available)
    DCEI.SetButtonFrameEnable(self.Button, item_available)
end

local function CustomizeStoreItem_Pack(self)
    local product_id = self.item_data.product_id
    if self.item_data.custom.full_price and self.FullCostLabel then
        -- check if localized price is USD
        local full_price_text = "¥" .. self.item_data.custom.full_price
        local full_price_id = self.item_data.custom.full_price_id
        local localized_full_price = shop.GetProductPriceLocalized(full_price_id)
        local localized_price = shop.GetProductPriceLocalized(product_id)
        local default_price = "¥" .. shop.GetProductPriceUSD(product_id)

        -- add "USD" suffix to full price display if localized price exists and isn't USD
        if localized_price and localized_price ~= default_price then
            full_price_text = full_price_text .. "<size=50%>" .. "CNY"
        end

        local text = localized_full_price or full_price_text
        DCEI.SetTextFrameText(self.FullCostLabel, text)
    end

    local stock_remaining = shop.GetItemStockAvailable(product_id)
    local item_available = stock_remaining > 0
    DCEI.SetFrameActive(self.PurchasedFrame, not item_available)
    DCEI.SetButtonFrameEnable(self.Button, item_available)
end

local function UpdateStoreItem_Chest(self)
    local product_id = self.item_data.product_id
    local custom_price_data = shop.GetProductCustomPriceData(product_id)
    local keys = GetBankData(custom_price_data.key_type)
    if keys > 0 then
        DCEI.SetFrameActive(self.KeyCostStack, true)
        DCEI.SetFrameActive(self.CostStack, false)
        DCEI.SetTextFrameText(self.KeyCostLabel, string.format("%i", keys) .. "/1")
        self.pip:ShowAnimated()
    else
        DCEI.SetFrameActive(self.KeyCostStack, false)
        DCEI.SetFrameActive(self.CostStack, true)
        self.pip:Hide()
    end
end

local function ManualUpdateStoreItem_Chest(product_id)
    local rune_chest_layout = shop.GetItemLayout(product_id)

    if rune_chest_layout and DCEI.FrameExists(rune_chest_layout.Frame) then
        UpdateStoreItem_Chest(rune_chest_layout)
    end
end

local function CustomizeStoreItem_Chest(self)
    self.pip = gmui.NewPip({
        parent = self.Background,
        anchor = "top_left",
    })
end

local function UpdateStoreItem_Card(self)
    --[[ local product_id = self.item_data.product_id
    local stock_remaining = shop.GetItemStockAvailable(product_id)
    local item_available = stock_remaining > 0
    DCEI.SetFrameActive(self.PurchasedFrame, not item_available)
    DCEI.SetButtonFrameEnable(self.Button, item_available) ]]

    local card_data = GetBankData("daily_reward_card")
    local days = DCEI.GetTranslationText("general/days")
    if self.item_data.product_id == "com.wildarmy.test.iap.bundle.goldcard" then
        if card_data.gold_card then
            DCEI.SetFrameActive(self.CardRemainingBanner, true)
            local card_duration = card_data.gold_card.expire_day - card_data.gold_card.last_claimed_day
            if card_duration == 1 then
                days = DCEI.GetTranslationText("general/day")
            end
            DCEI.SetTextFrameText(self.CardRemaining, card_duration .. " " .. days)
        else
            DCEI.SetFrameActive(self.CardRemainingBanner, false)
        end
    else
        -- gems or other daily items
    end
end

local function CustomizeStoreItem_ChestPopup(self)
    local product_id = self.item_data.product_id
    local custom_price_data = shop.GetProductCustomPriceData(product_id)
    local cost = custom_price_data.amount
    local key_type = custom_price_data.key_type
    if key_type then
        local keys = GetBankData(key_type)
        local key_icon
        if key_type == "gear_key_common" then
            key_icon = "cartoon_icon_mergewar_key1"
        else
            key_icon = "cartoon_icon_mergewar_key4_pocketquest"
        end

        if keys > 0 then
            DCEI.SetFrameImage(self.CostIcon, key_icon)
            DCEI.SetTextFrameText(self.CostLabel, string.format("%i", keys) .. "/1")
        else
            DCEI.SetFrameImage(self.CostIcon, "cartoon_icon_mergewar_gem2")
            DCEI.SetTextFrameText(self.CostLabel, FormatGoldCost(cost))
        end
    end

    local items = self.item_data:GetItemsData()
    local new_card_rate = 0
    if items.bigchest then
        new_card_rate = 5
    elseif items.bigestchest then
        new_card_rate = 20
    end

    local owned_cards, not_owned_cards = meta.GetPlayerOwnedAndNotOwnedCardsAvailable()
    if #not_owned_cards <= 0 then
        new_card_rate = 0
    end

    local color = new_card_rate > 0 and "<color=#ff78ff> <size=150%>" or " "
    local text = self.item_data.info.label
    text = text
        .. "\n\n"
        .. DCEI.GetTranslationText("meta/shop/new_card")
        .. color
        .. new_card_rate
        .. "%</size></color>"
    DCEI.SetTextFrameText(self.InfoLabel, text)
end

local function CustomizeStoreItem_ShardPack(self)
    CustomizeStoreItem_Pack(self)
    DCEI.SetTextFrameText(self.DiscountLabel, DCEI.GetTranslationText("shop/half_off", { x = "65%" }))
end

local function CustomizeStoreItem_Card(self)
    local icon = ""
    local values
    if self.item_data.product_id == "com.wildarmy.test.iap.bundle.goldcard" then
        icon = "<sprite name=cartoon_icon_mergewar_coin3>"
        values = meta.daily_card_data.gold_card
    else
        icon = "<sprite name=cartoon_icon_mergewar_gem2>"
        values = meta.daily_card_data.gem_card
    end
    local total_income = values.immediate + values.income * values.duration
    DCEI.SetTextFrameText(
        self.CardTotal,
        "<size=225%><sprite name=cartoon_icon_mergewar_coin3></size>" .. FormatGoldCost(total_income, 400)
    )
    DCEI.SetTextFrameText(
        self.CardImmediate,
        DCEI.GetTranslationText("shop/card/immediate", { x = icon .. " " .. FormatGoldCost(values.immediate, 125) })
    )
    DCEI.SetTextFrameText(
        self.CardIncome,
        DCEI.GetTranslationText(
            "shop/card/daily_amount",
            { x = icon .. " " .. FormatGoldCost(values.income, 125), y = values.duration }
        )
    )
end

local function CustomizeStoreItem_DailyAdWatch(self)
    self.pip = gmui.NewPip({
        parent = self.Background,
        anchor = "top_left",
    })

    if self.item_data.product_id == "com.wildarmy.test.shop.gift04" then
        local ads_card = GetBankData("ads_card")
        local ads_card_data = data.unit_data[ads_card]
        DCEI.SetFrameImage(self.Background, meta.shop_card_rarity[ads_card_data.rarity])
        DCEI.SetFrameActive(self.ItemImage, false)
        DCEI.SetFrameActive(self.PortraitContainer, true)
        if self.unit_name ~= ads_card then
            self.unit_name = ads_card
            if self.portrait and DCEI.FrameExists(self.portrait.Frame) then
                DCEI.DestroyFrame(self.portrait.Frame)
            end
            DCEI.UpdateFrame(self.PortraitContainer, function()
                self.portrait = SetUnitPortrait(self.PortraitContainer, ads_card_data)
            end)
        end
    end

    local ads_index = GetBankData("calender").ads_index or 1
    local last_item_index = (ads_index + 1) % 4 + 1
    if self.item_data.product_id == "com.wildarmy.test.shop.gift0" .. last_item_index then
        DCEI.SetFrameActive(self.NextArrow, false)
    end

    function self:Claimed()
        DCEI.SetFrameActive(self.Mask, true)
        DCEI.SetFrameActive(self.Tick, true)
        DCEI.SetFrameActive(self.Lock, false)
        DCEI.SetFrameActive(self.Timer, false)
        self.pip:Hide()
    end

    function self:ToBeClaimed()
        DCEI.SetFrameActive(self.Mask, false)
        DCEI.SetFrameActive(self.Tick, false)
        DCEI.SetFrameActive(self.Lock, false)
        if game_shop.shop_ads_item_ready then
            DCEI.SetFrameActive(self.Timer, false)
            self.pip:ShowAnimated()
        else
            self:Locked(true)
        end
    end

    function self:Locked(display_timer)
        DCEI.SetFrameActive(self.Mask, true)
        DCEI.SetFrameActive(self.Tick, false)
        DCEI.SetFrameActive(self.Lock, true)
        DCEI.SetFrameActive(self.Timer, display_timer)
        -- hook up timer label
        self.pip:Hide()
    end

    function self:UpdateState()
        local ads_index = GetBankData("calender").ads_index or 1
        local previous_index = ads_index - 1
        if previous_index <= 0 then
            previous_index = 4
        end
        DCEI.SetFrameActive(self.NextArrow, true)
        local last_item_index = (ads_index + 1) % 4 + 1
        if self.item_data.product_id == "com.wildarmy.test.shop.gift0" .. last_item_index then
            DCEI.SetFrameActive(self.NextArrow, false)
        end
        if self.item_data.product_id == "com.wildarmy.test.shop.gift0" .. ads_index then
            self:ToBeClaimed()
        else
            if self.item_data.product_id == "com.wildarmy.test.shop.gift0" .. previous_index then
                self:Claimed()
            else
                self:Locked()
            end
        end
    end

    self:Locked()
end

function game_shop.UpdateStoreWatchAdItems()
    local shop_state = GetBankData("shop_state")
    for i = 1, 4 do
        local product_id = "com.wildarmy.test.shop.gift0" .. i
        local layout = shop.GetItemLayout(product_id)
        if layout and DCEI.FrameExists(layout.Frame) then
            DCEI.SetFrameActive(layout.ForbiddenIcon, shop_state.remove_ads)
            layout:UpdateState()
        end
    end
end

local function UpdateShopAdsTimer(time)
    local index = GetBankData("calender").ads_index or 1
    local product_id = "com.wildarmy.test.shop.gift0" .. index
    local gift_layout = shop.GetItemLayout(product_id)
    if gift_layout and DCEI.FrameExists(gift_layout.Frame) then
        if time and time > 0 then
            local fillamount = 1 - time / meta.five_hour
            DCEI.SetTextFrameText(gift_layout.TimerLabel, FormatTimeShort(time))
            DCEI.SetFrameImageFillAmount(gift_layout.BarFill, fillamount)
            ui.AnimateClockHand(gift_layout)
        else
            DCEI.SetFrameActive(gift_layout.Timer, false)
        end
    end
end

local function CheckFirstTimeBonus(product_id)
    local bank_data = GetBankData("shop_1st_time_bonus")
    return not bank_data[product_id]
end

local function UpdateStoreGemDisplay(self)
    local is_first_time = CheckFirstTimeBonus(self.item_data.product_id)
    local gem_value = self.item_data:GetItemsData().gems
    if is_first_time then
        DCEI.SetFrameActive(self.first_bonus.Frame, true)
        DCEI.SetTextFrameText(self.first_bonus.ValueLabel, "+" .. gem_value)
    else
        DCEI.SetFrameActive(self.first_bonus.Frame, false)
    end
end

local CURRENCIES = {
    {
        name = "gold",
        icon = "cartoon_icon_mergewar_coin3",
        bundle_layout = "Shared/Shop/Section/Item/StarterBundleItem",
        bundle_image = "cartoon_icon_store_coin_level3_mergewar",
    },
    {
        name = "gems",
        icon = "cartoon_icon_mergewar_gem2",
        bundle_layout = "Shared/Shop/Section/Item/StarterBundleItem",
        bundle_image = "cartoon_icon_store_gem_level3_mergewar",
    },
    {
        name = "shard_dragon",
        icon = "",
        bundle_layout = "Shared/Shop/Section/Item/StarterBundleItemDragon",
    },
    {
        name = "shard_assassin",
        icon = "",
        bundle_layout = "Shared/Shop/Section/Item/StarterBundleItemAssassin",
    },
    {
        name = "shard_priest",
        icon = "",
        bundle_layout = "Shared/Shop/Section/Item/StarterBundleItemPriest",
    },
    {
        name = "shard_sage",
        icon = "",
        bundle_layout = "Shared/Shop/Section/Item/StarterBundleItemSage",
    },
    {
        name = "daily_ads",
        icon = "cartoon_icon_tv_mergewar",
    },
    {
        name = "_remove_ad",
    },
    {
        name = "gear_key_common",
        icon = "cartoon_icon_mergewar_key1",
    },
    {
        name = "gear_key_magic",
        icon = "cartoon_icon_mergewar_key4_pocketquest",
    },
    {
        name = "z1_booster_punch",
        icon = "cartoon_icon_gear_energy_punch_mergewar",
        bundle_layout = "Shared/Shop/Section/Item/BundleBooster",
    },
    {
        name = "z2_booster_bigbang",
        icon = "cartoon_icon_gear_bigbang_mergewar",
        bundle_layout = "Shared/Shop/Section/Item/BundleBooster",
    },
    {
        name = "talent_stone",
        icon = "fantasy_icon_talent_yellow_affinity_pocketshooter",
        bundle_layout = "Shared/Shop/Section/Item/BundleTalentStone",
    },
}

local IAP_LIST = {
    {
        product_id = "com.wildarmy.test.iap.dummy1499",
        price = {
            usd = 52,
        },
    },
    {
        product_id = "com.wildarmy.test.iap.dummy1999",
        price = {
            usd = 70,
        },
    },
    {
        product_id = "com.wildarmy.test.iap.dummy3999",
        price = {
            usd = 140,
        },
    },
    {
        product_id = "com.wildarmy.test.iap.dummy4999",
        price = {
            usd = 175,
        },
    },
    -- Starter Packs
    {
        product_id = "com.wildarmy.test.iap.bundle.starter01",
        label = DCEI.GetTranslationText("meta/shop/starter_pack"),
        display = {
            use_item_bundle = true,
            format_with_commas = true,
        },
        items = {
            gems = 1000,
            shard_dragon = 70,
        },
        price = {
            usd = 17,
        },
        stock_limit = 1,
        custom = {
            layout = "Portrait/Shop/Section/Item/Largest_StarterPack",
            layout_func = CustomizeStoreItem_Pack,
            update_func = UpdateStoreItem_Pack,
            full_price = 52,
            full_price_id = "com.wildarmy.test.iap.dummy1499",
        },
        reset_popup = true,
    },
    -- Elite Pack, appear after player buys starter pack
    {
        product_id = "com.wildarmy.test.iap.bundle.elite03",
        label = DCEI.GetTranslationText("meta/shop/elite"),
        display = {
            use_item_bundle = true,
            format_with_commas = true,
        },
        items = {
            gems = 1000,
            shard_priest = 100,
        },
        price = {
            usd = 35,
        },
        stock_limit = 1,
        custom = {
            layout = "Portrait/Shop/Section/Item/ElitePack",
            layout_func = CustomizeStoreItem_Pack,
            update_func = UpdateStoreItem_Pack,
            full_price = 175,
            full_price_id = "com.wildarmy.test.iap.dummy4999",
        },
        reset_popup = true,
    },
    -- Shard Pack
    {
        product_id = "com.wildarmy.test.iap.bundle.shard02",
        label = DCEI.GetTranslationText("shop/general/best_value"),
        display = {
            use_item_bundle = true,
            format_with_commas = true,
        },
        items = {
            gold = 3000,
            shard_sage = 200,
        },
        price = {
            usd = 24,
        },
        stock_limit = 1,
        custom = {
            layout = "Portrait/Shop/Section/Item/ShardBundle",
            layout_func = CustomizeStoreItem_ShardPack,
            update_func = UpdateStoreItem_Pack,
            full_price = 70,
            full_price_id = "com.wildarmy.test.iap.dummy1999",
        },
        reset_popup = true,
    },
    -- Gold Card
    {
        product_id = "com.wildarmy.test.iap.bundle.goldcard",
        label = DCEI.GetTranslationText("shop/card/gold"),
        display = {
            image = DCEI.Texture("cartoon_icon_gold_card_mergewar"),
        },
        price = {
            usd = 17,
        },
        custom = {
            layout = "Portrait/Shop/Section/Item/Card",
            layout_func = CustomizeStoreItem_Card,
            update_func = UpdateStoreItem_Card,
        },
        reset_popup = true,
    },
    {
        product_id = "com.wildarmy.test.iap.bundle.gemcard",
        label = DCEI.GetTranslationText("shop/card/gems"),
        display = {
            image = DCEI.Texture("general_icon_wildsky_monthly_card"),
        },
        price = {
            usd = 35,
        },
        custom = {
            layout = "Portrait/Shop/Section/Item/Card",
            layout_func = CustomizeStoreItem_Card,
            update_func = UpdateStoreItem_Card,
        },
    },
    -- remove ads
    {
        product_id = "com.wildarmy.test.iap.remove_ad.01",
        price = {
            usd = 52,
        },
        custom = {
            layout = "Portrait/Shop/Section/Item/RemoveAds",
            layout_func = CustomizeStoreItem_Pack,
            update_func = UpdateStoreItem_Pack,
        },
        items = {
            _remove_ad = 1,
            gold = 10000,
        },
        display = {
            use_stock_display = true,
            format_with_commas = true,
        },
        stock_limit = 1,
    },
    -- booster pack
    {
        product_id = "com.wildarmy.test.iap.bundle.booster01",
        label = DCEI.GetTranslationText("meta/shop/special_offer"),
        display = {
            use_item_bundle = true,
            format_with_commas = true,
        },
        items = {
            gold = 2000,
            z1_booster_punch = 10,
            z2_booster_bigbang = 10,
        },
        price = {
            usd = 35,
        },
        stock_limit = 1,
        custom = {
            layout = "Portrait/Shop/Section/Item/BoosterBundle",
            layout_func = CustomizeStoreItem_Pack,
            update_func = UpdateStoreItem_Pack,
            full_price = 140,
            full_price_id = "com.wildarmy.test.iap.dummy3999",
        },
        reset_popup = true,
    },
    -- talent stone pack, limited x2 reset weekly
    {
        product_id = "com.wildarmy.test.iap.bundle.talent_stone",
        type = "talent_stone",
        label = DCEI.GetTranslationText("meta/shop/talent_pack"),
        display = {
            use_item_bundle = true,
            format_with_commas = true,
        },
        items = {
            gold = 1500,
            talent_stone = 10,
        },
        price = {
            usd = 17,
        },
        stock_limit = 1,
        custom = {
            layout = "Portrait/Shop/Section/Item/TalentStoneBundle",
            layout_func = CustomizeStoreItem_Pack,
            update_func = UpdateStoreItem_Pack,
        },
    },
    {
        product_id = "com.wildarmy.test.iap.bundle.talent_stone02",
        type = "talent_stone",
        label = DCEI.GetTranslationText("meta/shop/talent_pack"),
        display = {
            use_item_bundle = true,
            format_with_commas = true,
        },
        items = {
            gold = 3500,
            talent_stone = 20,
        },
        price = {
            usd = 35,
        },
        stock_limit = 1,
        custom = {
            layout = "Portrait/Shop/Section/Item/TalentStoneBundle",
            layout_func = CustomizeStoreItem_Pack,
            update_func = UpdateStoreItem_Pack,
        },
    },
    {
        product_id = "com.wildarmy.test.iap.bundle.talent_stone03",
        type = "talent_stone",
        label = DCEI.GetTranslationText("meta/shop/talent_pack"),
        display = {
            use_item_bundle = true,
            format_with_commas = true,
        },
        items = {
            gold = 8000,
            talent_stone = 40,
        },
        price = {
            usd = 70,
        },
        stock_limit = 1,
        custom = {
            layout = "Portrait/Shop/Section/Item/TalentStoneBundle",
            layout_func = CustomizeStoreItem_Pack,
            update_func = UpdateStoreItem_Pack,
        },
    },
    -- Gifts
    {
        product_id = "com.wildarmy.test.shop.gift01",
        label = {
            icon = "cartoon_icon_mergewar_gem2",
            item_type = "gems",
            format_with_commas = true,
        },
        display = {
            image = "cartoon_icon_store_gem_level1_mergewar",
            scale = 0.8,
            offset = { y = 10 },
        },
        items = {
            gems = 20,
        },
        price = {
            type = "daily_ads",
            amount = 1,
        },
        custom = {
            layout = "Portrait/Shop/Section/Item/DailyAdWatch",
            layout_func = CustomizeStoreItem_DailyAdWatch,
        },
    },
    {
        product_id = "com.wildarmy.test.shop.gift02",
        label = {
            icon = "cartoon_icon_mergewar_coin3",
            item_type = "gold",
            format_with_commas = true,
        },
        display = "cartoon_icon_store_coin_level2_mergewar",
        items = {
            gold = 500,
        },
        price = {
            type = "daily_ads",
            amount = 1,
        },
        custom = {
            layout = "Portrait/Shop/Section/Item/DailyAdWatch",
            layout_func = CustomizeStoreItem_DailyAdWatch,
        },
    },
    {
        product_id = "com.wildarmy.test.shop.gift03",
        label = DCEI.GetTranslationText("meta/shop/rare_chest"),
        display = {
            image = "cartoon_icon_mergewar_chest_card_rank2_outline",
            scale = 1.2,
            offset = { y = 20 },
        },
        items = {
            chest = 1,
        },
        price = {
            type = "daily_ads",
            amount = 1,
        },
        custom = {
            layout = "Portrait/Shop/Section/Item/DailyAdWatch",
            layout_func = CustomizeStoreItem_DailyAdWatch,
        },
    },
    {
        product_id = "com.wildarmy.test.shop.gift04",
        label = {
            icon = "cartoon_icon_mergewar_card",
            name = "x5",
        },
        -- display = "cartoon_icon_mergewar_card",
        items = {
            ads_card = 5,
        },
        price = {
            type = "daily_ads",
            amount = 1,
        },
        custom = {
            layout = "Portrait/Shop/Section/Item/DailyAdWatch",
            layout_func = CustomizeStoreItem_DailyAdWatch,
        },
    },
    -- Gems
    {
        product_id = "com.wildarmy.test.iap.gems01",
        type = "gem",
        label = {
            icon = "cartoon_icon_mergewar_gem2",
            item_type = "gems",
            format_with_commas = true,
        },
        display = {
            image = DCEI.Texture("cartoon_icon_store_gem_level1_mergewar"),
            scale = 0.76,
            offset = { y = 20 },
        },
        items = {
            gems = 80,
        },
        price = {
            usd = 3,
        },
        custom = {
            layout = "Portrait/Shop/Section/Item/StandardGems",
            update_func = UpdateStoreGemDisplay,
        },
    },
    {
        product_id = "com.wildarmy.test.iap.gems02",
        type = "gem",
        label = {
            icon = "cartoon_icon_mergewar_gem2",
            item_type = "gems",
            format_with_commas = true,
        },
        display = {
            image = DCEI.Texture("cartoon_icon_store_gem_level2_mergewar"),
            offset = { y = 30 },
        },
        items = {
            gems = 500,
        },
        price = {
            usd = 17,
        },
        custom = {
            layout = "Portrait/Shop/Section/Item/StandardGems",
            update_func = UpdateStoreGemDisplay,
        },
    },
    {
        product_id = "com.wildarmy.test.iap.gems03",
        type = "gem",
        label = {
            icon = "cartoon_icon_mergewar_gem2",
            item_type = "gems",
            format_with_commas = true,
        },
        display = {
            image = DCEI.Texture("cartoon_icon_store_gem_level3_mergewar"),
            scale = 1.1,
            offset = { y = 20 },
        },
        items = {
            gems = 1200,
        },
        price = {
            usd = 35,
        },
        custom = {
            layout = "Portrait/Shop/Section/Item/StandardGems",
            update_func = UpdateStoreGemDisplay,
        },
    },
    {
        product_id = "com.wildarmy.test.iap.gems04",
        type = "gem",
        label = {
            icon = "cartoon_icon_mergewar_gem2",
            item_type = "gems",
            format_with_commas = true,
        },
        display = {
            image = DCEI.Texture("cartoon_icon_store_gem_level4_mergewar"),
            scale = 1.1,
            offset = { y = 15 },
        },
        items = {
            gems = 2500,
        },
        price = {
            usd = 70,
        },
        custom = {
            layout = "Portrait/Shop/Section/Item/StandardGems",
            update_func = UpdateStoreGemDisplay,
        },
    },
    {
        product_id = "com.wildarmy.test.iap.gems05",
        type = "gem",
        label = {
            icon = "cartoon_icon_mergewar_gem2",
            item_type = "gems",
            format_with_commas = true,
        },
        display = {
            image = DCEI.Texture("cartoon_icon_store_gem_level5_mergewar"),
            scale = 1.2,
            offset = { y = 20 },
        },
        items = {
            gems = 6500,
        },
        price = {
            usd = 175,
        },
        custom = {
            layout = "Portrait/Shop/Section/Item/StandardGems",
            update_func = UpdateStoreGemDisplay,
        },
    },
    {
        product_id = "com.wildarmy.test.iap.gems06",
        type = "gem",
        label = {
            icon = "cartoon_icon_mergewar_gem2",
            item_type = "gems",
            format_with_commas = true,
        },
        display = {
            image = DCEI.Texture("cartoon_icon_store_gem_level6_mergewar"),
            scale = 1.2,
            offset = { y = 20 },
        },
        items = {
            gems = 14000,
        },
        price = {
            usd = 935,
        },
        custom = {
            layout = "Portrait/Shop/Section/Item/StandardGems",
            update_func = UpdateStoreGemDisplay,
        },
    },
    -- Gold
    {
        product_id = "com.wildarmy.test.shop.gold01",
        label = {
            icon = "cartoon_icon_mergewar_coin3",
            item_type = "gold",
            format_with_commas = true,
        },
        display = DCEI.Texture("cartoon_icon_store_coin_level1_mergewar"),
        items = {
            gold = 1000,
        },
        price = {
            type = "gems",
            amount = 60,
        },
    },
    {
        product_id = "com.wildarmy.test.shop.gold02",
        label = {
            icon = "cartoon_icon_mergewar_coin3",
            item_type = "gold",
            format_with_commas = true,
        },
        display = {
            image = DCEI.Texture("cartoon_icon_store_coin_level2_mergewar"),
            scale = 1.1,
        },
        items = {
            gold = 10000,
        },
        price = {
            type = "gems",
            amount = 500,
        },
    },
    {
        product_id = "com.wildarmy.test.shop.gold03",
        label = {
            icon = "cartoon_icon_mergewar_coin3",
            item_type = "gold",
            format_with_commas = true,
        },
        display = {
            image = DCEI.Texture("cartoon_icon_store_coin_level3_mergewar"),
            scale = 1.2,
            offset = { y = 10 },
        },
        items = {
            gold = 100000,
        },
        price = {
            type = "gems",
            amount = 4500,
        },
    },
    -- Gacha
    {
        product_id = "com.wildarmy.test.shop.chest01",
        label = {
            name = DCEI.GetTranslationText("meta/shop/rare_chest"),
        },
        display = {
            image = "cartoon_icon_mergewar_chest_card_rank2_outline",
        },
        info = {
            update_func = CustomizeStoreItem_ChestPopup,
            title = DCEI.GetTranslationText("meta/shop/rare_chest"),
            label = DCEI.GetTranslationText("meta/shop/chest_content", { x = "10" }),
        },
        items = {
            chest = 1,
        },
        price = {
            type = "gems",
            amount = 60,
            key_type = "gear_key_common",
        },
        custom = {
            layout = "Portrait/Shop/Section/Item/StandardChestTeal",
            layout_func = CustomizeStoreItem_Chest,
            update_func = UpdateStoreItem_Chest,
        },
    },
    {
        product_id = "com.wildarmy.test.shop.chest02",
        label = {
            name = DCEI.GetTranslationText("meta/shop/epic_chest"),
        },
        display = {
            image = "cartoon_icon_mergewar_chest_card_rank3_outline",
        },
        info = {
            update_func = CustomizeStoreItem_ChestPopup,
            title = DCEI.GetTranslationText("meta/shop/epic_chest"),
            label = DCEI.GetTranslationText(
                "meta/shop/epic_chest_content",
                { x = "<size=120%><color=yellow>60</color></size>" }
            ),
        },
        items = {
            bigchest = 1,
        },
        price = {
            type = "gems",
            amount = 250,
            key_type = "gear_key_magic",
        },
        custom = {
            layout = "Portrait/Shop/Section/Item/StandardChestPurple",
            layout_func = CustomizeStoreItem_Chest,
            update_func = UpdateStoreItem_Chest,
        },
    },
    {
        product_id = "com.wildarmy.test.shop.chest03",
        label = {
            name = DCEI.GetTranslationText("meta/shop/mystic_chest"),
        },
        display = {
            image = "cartoon_icon_mergewar_chest_card_rank4_outline",
        },
        info = {
            update_func = CustomizeStoreItem_ChestPopup,
            title = DCEI.GetTranslationText("meta/shop/mystic_chest"),
            label = DCEI.GetTranslationText(
                "meta/shop/legendary_chest_content",
                { x = "<size=120%><color=yellow>300</color></size>", y = "<size=120%><color=yellow>2</color></size>" }
            ),
        },
        items = {
            bigestchest = 1,
        },
        price = {
            type = "gems",
            amount = 1000,
        },
        custom = {
            layout = "Portrait/Shop/Section/Item/StandardChestYellow",
        },
    },
    {
        product_id = "com.wildarmy.test.shop.healing_elixir",
        type = "elixir",
        label = {
            name = DCEI.GetTranslationText("meta/shop/healing_elixir"), -- "Healing Elixir",
            color = "<color=#42cdff>",
        },
        items = {
            ["Healing Elixir"] = 1,
        },
        price = {
            type = "gold",
            amount = 100,
        },
    },
    {
        product_id = "com.wildarmy.test.shop.ironskin_elixir",
        type = "elixir",
        label = {
            name = DCEI.GetTranslationText("meta/shop/ironskin_elixir"), -- "Ironskin Elixir",
            color = "<color=#42cdff>",
        },
        items = {
            ["Ironskin Elixir"] = 1,
        },
        price = {
            type = "gold",
            amount = 20000,
        },
    },
    {
        product_id = "com.wildarmy.test.shop.berserker_elixir",
        type = "elixir",
        label = {
            name = DCEI.GetTranslationText("meta/shop/berserker_elixir"), -- "Berserker Elixir",
            color = "<color=#42cdff>",
        },
        items = {
            ["Berserker Elixir"] = 1,
        },
        price = {
            type = "gems",
            amount = 1500,
        },
    },
    {
        product_id = "com.wildarmy.test.shop.vampiric_elixir",
        type = "elixir",
        label = {
            name = DCEI.GetTranslationText("meta/shop/vampiric_elixir"), -- "Vampiric Elixir",
            color = "<color=#42cdff>",
        },
        items = {
            ["Vampiric Elixir"] = 1,
        },
        price = {
            type = "gems",
            amount = 2000,
        },
    },
    {
        product_id = "com.wildarmy.test.shop.toxic_elixir",
        type = "elixir",
        label = {
            name = DCEI.GetTranslationText("meta/shop/toxic_elixir"), -- "Toxic Elixir",
            color = "<color=#42cdff>",
        },
        items = {
            ["Toxic Elixir"] = 1,
        },
        price = {
            usd = 17,
        },
    },
    {
        --resource_bundle
        product_id = "com.wildarmy.test.iap.resource_bundle.01",
        price = {
            usd = 17,
        },
        custom = {
            layout = "Meta/Page5/Bundle",
        },
    },
    {
        --Hand_of_midas
        product_id = "com.wildarmy.test.iap.hand_of_midas.01",
        price = {
            usd = 24,
        },
        custom = {
            layout = "Meta/Page5/Bundle",
        },
    },
    {
        -- live event bunlde
        product_id = "com.wildarmy.test.iap.bundle.live_event01",
        price = {
            usd = 24,
        },
        items = {
            gem = 320,
            unit_shards = 30,
            event_key = 70,
        },
        custom = {
            full_price = 140,
            full_price_id = "com.wildarmy.test.iap.dummy3999",
        },
    },
    {
        -- live event pass bundle
        product_id = "com.wildarmy.test.iap.bundle.live_event_pass01",
        price = {
            usd = 35,
        },
        items = {
            gold = 10000,
            event_pass = 1,
        },
    },
}

local function CustomizeStoreSection_Deals(self)
    -- animate sparkles
    for n = 1, 6 do
        local key = "Sparkle0" .. n
        local sparkle = self[key]
        ui.AnimateSparkle(sparkle)
    end
end

local function GenerateShopAdsCard()
    local owned_cards, not_owned_cards = meta.GetPlayerOwnedAndNotOwnedCardsAvailable()
    local rd = math.random(#owned_cards - 3, #owned_cards)
    CommitBankData("ads_card", owned_cards[rd].name)
end

local function GenerateAdsCardsChest()
    local chest = {}
    local card_name = GetBankData("ads_card")
    table.insert(chest, { name = card_name, num = 5 })
    chest.chest_unit = DCEI.Unit("ChestSilver")
    chest.type = "card"

    return chest
end

local function AttemptClaimShopAdsItem(product_id)
    if meta.CheckCreateRemoveAdsPromotion() then
        return
    end
    local server_time
    local function ResolveAds(success)
        if success and server_time then
            local bank_calender = GetBankData("calender")
            local item_index = bank_calender.ads_index or 1
            bank_calender.ads_index = item_index % 4 + 1
            bank_calender.shop_ads_time = server_time

            game_shop.shop_ads_item_ready = false
            game_shop.AttemptStartShopAdsTimer(meta.five_hour)
            game_shop.UpdateStoreWatchAdItems()
            CommitBankData("calender")

            if product_id == "com.wildarmy.test.shop.gift01" then
                LogEvent(LogEvents.WatchAds("ShopGem"))
                menu.pages[1]:StoreWatchAdItemRotateAnimation()
            elseif product_id == "com.wildarmy.test.shop.gift02" then
                LogEvent(LogEvents.WatchAds("ShopGold"))
                menu.pages[1]:StoreWatchAdItemRotateAnimation()
            elseif product_id == "com.wildarmy.test.shop.gift03" then
                LogEvent(LogEvents.WatchAds("ShopChest"))
                game_shop.need_rotate_dailyads = true
            else
                LogEvent(LogEvents.WatchAds("ShopCards"))
                game_shop.need_rotate_dailyads = true
            end
            local currency = "ad"
            game_shop.OnAnyItemPurchase(product_id, currency)

            if item_index == 4 then
                GenerateShopAdsCard()
            end

            game_shop.CheckShopButtonPip()
            local bank_tutorial = GetBankData("tutorial_state")
            if not bank_tutorial.campaign_gift then
                bank_tutorial.campaign_gift = true
                CommitBankData("tutorial_state")
                DCEI.Event.LogTutorialComplete("Free Gift")
            end
        else
            local text = DCEI.GetTranslationText("meta/failed")
            ui.FeedbackMessage(text)
        end
    end

    local function CheckRemoveAds(time_stamp)
        if time_stamp and time_stamp > 0 then
            server_time = time_stamp
            local bank_calender = GetBankData("calender")
            -- validate timer
            -- how much time elapsed from last shop ads timer start point and calculate rest time
            bank_calender.shop_ads_time = bank_calender.shop_ads_time or 0
            local ads_timer_remaining = meta.five_hour - (time_stamp - bank_calender.shop_ads_time)
            -- 5 seconds buffer time
            if ads_timer_remaining > 5 then
                game_shop.shop_ads_item_ready = false
                game_shop.AttemptStartShopAdsTimer(ads_timer_remaining)
                game_shop.UpdateStoreWatchAdItems()
                return
            end
            -- claiming reward
            local shop_state = GetBankData("shop_state")
            if shop_state.remove_ads then
                ResolveAds(true)
            else                
                DCEI.ShowAds(ResolveAds)
            end
        else
            local text = DCEI.GetTranslationText("ui/meta/networkerror")
            ui.FeedbackMessage(text)
        end
    end

    local item_index = GetBankData("calender").ads_index or 1
    local available_product_id = "com.wildarmy.test.shop.gift0" .. item_index
    if product_id == available_product_id then
        if game_shop.shop_ads_item_ready then
            ui.CreateConnectingServerScreen()
            game.AttemptGetServerTime(CheckRemoveAds)
        else
            local text = DCEI.GetTranslationText("meta/not_ready_general")
            ui.FeedbackMessage(text)
        end
    end
end

function game_shop.InitializeShop(parent)
    shop.InitializeIAP({
        on_purchase_attempt_callback = game_shop.AttemptAnyItemPurchase,
        on_purchase_success_callback = game_shop.OnAnyItemPurchase,
        product_list = IAP_LIST,
        currency_list = CURRENCIES,
        is_game_localized = true,
    })

    local shop_page = shop.NewShopUI({ parent = parent, is_landscape = false })

    local pop_up = shop.NewPopupUI({ parent = ui.meta })
    pop_up.OnShowAnimation = gmui.Animations.BackIn

    function pop_up:OnShow()
        DCEI.TriggerAddTimerEventElapsed(function()
            ui.back_button_close_func = function()
                pop_up:Hide()
            end
        end, 0, true, true)
    end
    shop_page.pop_up = pop_up
    shop_page.popup_list = {}

    local shop_data = {
        {
            name = "bundles",
            display_name = DCEI.GetTranslationText("meta/shop/pack"),
            custom = {
                layout = "Portrait/Shop/Section_Deals",
                layout_func = CustomizeStoreSection_Deals,
            },
        },
        {
            name = "goldcard",
            custom = {
                layout = "Portrait/Shop/Section_Invisible",
            },
        },
        {
            name = "removeads",
            custom = {
                layout = "Portrait/Shop/Section_Invisible",
            },
        },
        {
            name = "boosters",
            custom = {
                layout = "Portrait/Shop/Section_Invisible",
            },
        },
        {
            name = "talent_stone",
            custom = {
                layout = "Portrait/Shop/Section_Invisible",
            },
            -- rows = {
            --     {
            --         "com.wildarmy.test.iap.bundle.talent_stone",
            --     },
            --     {
            --         "com.wildarmy.test.iap.bundle.talent_stone02",
            --     },
            --     {
            --         "com.wildarmy.test.iap.bundle.talent_stone03",
            --     },
            -- }
        },
        {
            name = "daily",
            display_name = DCEI.GetTranslationText("meta/shop/gift"),
            custom = {
                layout = "Portrait/Shop/Section_DailyAds",
            },
            -- rows = {
            -- {
            --     "com.wildarmy.test.shop.gift01",
            --     "com.wildarmy.test.shop.gift02",
            --     "com.wildarmy.test.shop.gift03",
            --     "com.wildarmy.test.shop.gift04",
            -- }
            -- }
        },
        {
            name = "chests",
            display_name = DCEI.GetTranslationText("meta/shop/chest"),
            rows = {
                {
                    "com.wildarmy.test.shop.chest01",
                    "com.wildarmy.test.shop.chest02",
                    "com.wildarmy.test.shop.chest03",
                },
            },
        },
        {
            name = "gems",
            display_name = DCEI.GetTranslationText("meta/shop/gem"),
            rows = {
                {
                    "com.wildarmy.test.iap.gems01",
                    "com.wildarmy.test.iap.gems02",
                    "com.wildarmy.test.iap.gems03",
                },
                {
                    "com.wildarmy.test.iap.gems04",
                    "com.wildarmy.test.iap.gems05",
                    "com.wildarmy.test.iap.gems06",
                },
            },
        },
        {
            name = "gold",
            display_name = DCEI.GetTranslationText("meta/shop/gold"),
            rows = {
                {
                    "com.wildarmy.test.shop.gold01",
                    "com.wildarmy.test.shop.gold02",
                    "com.wildarmy.test.shop.gold03",
                },
            },
        },
    }

    shop_page:Initialize(shop_data)

    local bundles = {
        "com.wildarmy.test.iap.bundle.starter01",
        "com.wildarmy.test.iap.bundle.elite03",
        "com.wildarmy.test.iap.bundle.shard02",
    }

    local options = { init = true }
    for _, product_id in ipairs(bundles) do
        local stock_remaining = shop.GetItemStockAvailable(product_id)
        if stock_remaining > 0 then
            shop_page:AddItemsToSection("bundles", { product_id }, options)
        end
    end

    shop_page:AddItemsToSection("goldcard", { "com.wildarmy.test.iap.bundle.goldcard" }, options)
    local product_id = "com.wildarmy.test.iap.remove_ad.01"
    local stock_remaining = shop.GetItemStockAvailable(product_id)
    if stock_remaining > 0 then
        shop_page:AddItemsToSection("removeads", { product_id }, options)
    end

    local product_id = "com.wildarmy.test.iap.bundle.booster01"
    local stock_remaining = shop.GetItemStockAvailable(product_id)
    if stock_remaining > 0 then
        shop_page:AddItemsToSection("boosters", { product_id }, options)
    end

    function shop_page:UpdateTalentStoneItems(init)
        local talent_stones = {
            "com.wildarmy.test.iap.bundle.talent_stone",
            "com.wildarmy.test.iap.bundle.talent_stone02",
            "com.wildarmy.test.iap.bundle.talent_stone03",
        }

        if not init then
            shop_page:RemoveAllItemsFromSection("talent_stone")
        end

        for _, bundle_id in ipairs(talent_stones) do
            local stock_remaining = shop.GetItemStockAvailable(bundle_id)
            if stock_remaining > 0 then
                shop_page:AddItemsToSection("talent_stone", { bundle_id }, options)
                break
            end
        end
    end

    shop_page:UpdateTalentStoneItems(true)

    function shop_page:RefreshStoreWatchAdItem()
        local ads_item_index = GetBankData("calender").ads_index or 1
        if shop_page.first_ads_index ~= ads_item_index then
            local section_name = "daily"
            self:RemoveAllItemsFromSection(section_name)
            local list = {}
            for i = 0, 2 do
                local index = (ads_item_index + i - 1) % 4 + 1
                local product_id = "com.wildarmy.test.shop.gift0" .. index
                table.insert(list, product_id)
            end
            self:AddItemsToSection(section_name, list, { row = 1 })
            shop_page.first_ads_index = ads_item_index
        end
    end

    function shop_page:StoreWatchAdItemRotateAnimation()
        game_shop.need_rotate_dailyads = false
        local section_name = "daily"
        local ads_item_index = GetBankData("calender").ads_index or 1
        local last_index = ads_item_index % 4 + 1
        local last_product_id = "com.wildarmy.test.shop.gift0" .. last_index
        local last_product_layout = shop.GetItemLayout(last_product_id)
        if last_product_layout and DCEI.FrameExists(last_product_layout.Frame) then
            DCEI.SetFrameActive(last_product_layout.NextArrow, true)
        end
        local new_added_index = (ads_item_index + 1) % 4 + 1
        local new_added_product_id = "com.wildarmy.test.shop.gift0" .. new_added_index
        self:AddItemsToSection(section_name, { new_added_product_id }, { row = 1 })
        BlockInput()
        DCEI.TriggerAddTimerEventElapsed(function()
            local new_added_product_layout = shop.GetItemLayout(new_added_product_id)
            DCEI.SetFrameActive(new_added_product_layout.NextArrow, false)
            local daily_ads_section = self.sections["daily"]
            local ease = "OutSine"
            DCEI.AnimateFrameScrollPosition(daily_ads_section.ItemContainer, 0, 256, 1.5, ease)
            DCEI.TriggerAddTimerEventElapsed(function()
                DCEI.AnimateFrameScrollPosition(daily_ads_section.ItemContainer, 0, 0, 0, "Linear")
                local previous_index = ads_item_index - 1
                if previous_index <= 0 then
                    previous_index = 4
                end
                local moved_out_product_id = "com.wildarmy.test.shop.gift0" .. previous_index
                local moved_out_layout = shop.GetItemLayout(moved_out_product_id)
                moved_out_layout:Remove()
                self.first_ads_index = ads_item_index
                UnblockInput()
            end, 1.5, true, true)
        end, 0.5, true, true)
    end

    function shop_page:OnShow()
        DCEI.SetFrameActive(self.CloseButton, false)
        self:RefreshStoreWatchAdItem()
        if game_shop.shop_ads_timer then
            game_shop.shop_ads_timer:Tick()
        end
        game_shop.UpdateStoreWatchAdItems()
        ManualUpdateStoreItem_Chest("com.wildarmy.test.shop.chest01")
        ManualUpdateStoreItem_Chest("com.wildarmy.test.shop.chest02")

        if
            shop.GetItemStockAvailable("com.wildarmy.test.iap.bundle.elite03") > 0
            and shop.GetItemStockAvailable("com.wildarmy.test.iap.bundle.starter01") > 0
        then
            shop.GetItemLayout("com.wildarmy.test.iap.bundle.elite03"):Hide()
        elseif
            shop.GetItemStockAvailable("com.wildarmy.test.iap.bundle.starter01") <= 0
            and shop.GetItemLayout("com.wildarmy.test.iap.bundle.starter01")
        then
            shop.GetItemLayout("com.wildarmy.test.iap.bundle.starter01"):Hide()
        end

        local item_time = shop.GetItemLazyLoadTimeRemaining() or 0
        DCEI.TriggerAddTimerEventElapsed(function()
            if shop.GetItemStockAvailable("com.wildarmy.test.iap.bundle.booster01") > 0 then
                local item_layout = shop.GetItemLayout("com.wildarmy.test.iap.bundle.booster01")
                if meta.CheckLevelClear(1, 15) and item_layout and DCEI.FrameExists(item_layout.Frame) then
                    shop.GetItemLayout("com.wildarmy.test.iap.bundle.booster01"):Show()
                else
                    shop.GetItemLayout("com.wildarmy.test.iap.bundle.booster01"):Hide()
                end
            end
        end, item_time, true, true)
    end

    function shop_page:OnHide()
        DCEI.SetOnClickCallback(menu.pages[1].ReturnButton, function() end)
        DCEI.SetFrameActive(menu.pages[1].ReturnButton, false)
    end

    shop_page.pop_up.OnHide = function()
        game_shop.CheckShopPopup()
        ui.back_button_close_func = nil
    end
    return shop_page
end

function game_shop.AttemptAnyItemPurchase(product_id, parent)
    DCEI.LogMessage("> Attempt Purchase " .. tostring(product_id))

    local custom_price_data = shop.GetProductCustomPriceData(product_id)

    local parent = parent or menu.pages[1]

    if custom_price_data then
        if custom_price_data.type == "gems" then
            if custom_price_data.key_type and GetBankData(custom_price_data.key_type) > 0 then
                CommitBankData(custom_price_data.key_type, GetBankData(custom_price_data.key_type) - 1)
                local currency = "key"
                ManualUpdateStoreItem_Chest(product_id)
                DCEI.WaitForUiUpdateToComplete(function()
                    game_shop.OnAnyItemPurchase(product_id, currency)
                end)
            else
                -- for currency gems
                local cost = custom_price_data.amount
                if GetBankData("gem") >= cost then
                    -- if player can afford purchase, subtract cost and deliver items
                    meta.AddGem(nil, -1 * cost)
                    local currency = "gem"
                    game_shop.OnAnyItemPurchase(product_id, currency)
                else
                    meta.NotEnoughGem(parent)
                end
            end
        elseif custom_price_data.type == "daily_ads" then
            AttemptClaimShopAdsItem(product_id)
        elseif custom_price_data.type == "gold" then
            local cost = custom_price_data.amount
            if GetBankData("gold") >= cost then
                meta.AddGold(nil, -1 * cost)
                local currency = "gold"
                game_shop.OnAnyItemPurchase(product_id, currency)
            else
                -- param 3 specifically handles elixir panel
                local item_data = shop.GetProductData(product_id)
                if item_data.type == "elixir" then
                    meta.NotEnoughGold(parent, 3)
                else
                    meta.NotEnoughGold(parent, 2)
                end
            end
        end
    else
        menu.pages[1].popup_list = {}
        -- otherwise attempt standard IAP flow
        if PlayingEditorMode() then
            -- simulate successful purchase when using editor
            shop.OnAnyItemPurchase(product_id)
        else
            -- otherwise continue standard IAP flow
            -- note: DCEI.PurchaseIapProduct() takes a few seconds to execute and thus acts similarly to a DCEI.Wait() so avoid subsequent code in the same thread
           -- DCEI.PurchaseIapProduct(product_id)
		  DCEI.TriggerAddMappedKeyDownEvent(product_id,shop.OnAnyItemPurchase)   
        end
    end
end

function game_shop.OnAnyItemPurchase(product_id, currency)
    DCEI.LogMessage("> Successful Purchase " .. tostring(product_id))
    menu.pages[1].pop_up:Hide()
    -- play item purchase sound
    meta.PlayGoldSound()

    local item_data = shop.GetProductData(product_id)
    local items = item_data:GetItemsData()
    -- deliver purchased items to player
    if item_data.product_id == "com.wildarmy.test.iap.bundle.starter01" then
        local starter_pack_chest = {
            { name = "gem", num = items.gems },
            { name = "Unit Dragon", num = items.shard_dragon },
            chest_unit = "ChestGold",
            type = "card",
        }
        starter_pack_chest.on_close = function()
            local starter_layout = shop.GetItemLayout("com.wildarmy.test.iap.bundle.starter01")
            if starter_layout then
                starter_layout:Hide()
            end
            DCEI.TriggerAddTimerEventElapsed(function()
                if shop.GetItemStockAvailable("com.wildarmy.test.iap.bundle.elite03") > 0 then
                    local elite_layout = shop.GetItemLayout("com.wildarmy.test.iap.bundle.elite03")
                    if elite_layout then
                        elite_layout:Show()
                    end
                end
            end, 0, false, true)
        end
        meta.OpenChest(starter_pack_chest)
    elseif item_data.product_id == "com.wildarmy.test.iap.bundle.elite03" then
        local elite_pack_chest = {
            { name = "gem", num = items.gems },
            { name = "Unit Priest", num = items.shard_priest },
            chest_unit = "ChestGold",
            type = "card",
        }
        meta.OpenChest(elite_pack_chest)
        local layout = shop.GetItemLayout(item_data.product_id)
        if layout then
            layout:Hide()
            if shop.GetItemStockAvailable("com.wildarmy.test.iap.bundle.shard02") == 0 then
                menu.pages[1].sections["bundles"]:Hide()
            end
        end
    elseif item_data.product_id == "com.wildarmy.test.iap.bundle.shard02" then
        local shard_chest = {
            { name = "gold", num = items.gold },
            { name = "Unit Sage", num = items.shard_sage },
            chest_unit = "ChestGold",
            type = "card",
        }
        meta.OpenChest(shard_chest)
        local layout = shop.GetItemLayout(item_data.product_id)
        if layout then
            layout:Hide()
            if shop.GetItemStockAvailable("com.wildarmy.test.iap.bundle.elite03") == 0 then
                menu.pages[1].sections["bundles"]:Hide()
            end
        end
    elseif
        item_data.product_id == "com.wildarmy.test.iap.bundle.goldcard"
        or item_data.product_id == "com.wildarmy.test.iap.bundle.gemcard"
    then
        local bank_calender = GetBankData("calender")
        local card_data = GetBankData("daily_reward_card")
        local current_day = bank_calender.day
        if item_data.product_id == "com.wildarmy.test.iap.bundle.goldcard" then
            if card_data.gold_card then
                card_data.gold_card.expire_day = card_data.gold_card.expire_day
                    + meta.daily_card_data.gold_card.duration
            else
                card_data.gold_card = {
                    expire_day = current_day + meta.daily_card_data.gold_card.duration - 1,
                    last_claimed_day = current_day - 1,
                }
            end
            local start_ui = shop.GetItemLayout(item_data.product_id) and shop.GetItemLayout(item_data.product_id).Frame
                or ui.root
            meta.AddGold(start_ui, meta.daily_card_data.gold_card.immediate)
        elseif item_data.product_id == "com.wildarmy.test.iap.bundle.gemcard" then
            if card_data.gem_card then
                card_data.gem_card.expire_day = card_data.gem_card.expire_day + meta.daily_card_data.gem_card.duration
            else
                card_data.gem_card = {
                    expire_day = current_day + meta.daily_card_data.gem_card.duration - 1,
                    last_claimed_day = current_day - 1,
                }
            end
            local start_ui = shop.GetItemLayout(item_data.product_id) and shop.GetItemLayout(item_data.product_id).Frame
                or ui.root
            meta.AddGem(start_ui, meta.daily_card_data.gem_card.immediate)
        end
        CommitBankData("daily_reward_card")
        meta.HandleDailyCardRewards()
        local item_layout = shop.GetItemLayout(item_data.product_id)
        if item_layout and DCEI.FrameExists(item_layout.Frame) then
            UpdateStoreItem_Card(item_layout)
        end
        local message =
            DCEI.GetTranslationText("ui/purchase_success", { x = DCEI.GetTranslationText("shop/card/gold") })
        ui.FeedbackMessage(message)
    elseif item_data.product_id == "com.wildarmy.test.iap.bundle.booster01" then
        local booster_chest = {
            { name = "gold", num = items.gold },
            { name = "booster", booster_name = "booster_punch", num = items.z1_booster_punch },
            { name = "booster", booster_name = "booster_bigbang", num = items.z2_booster_bigbang },
            chest_unit = "ChestPlatinum",
        }
        meta.OpenChest(booster_chest)
        if ui.game then
            ui.game:UpdateInGameBoosters(data.boosters)
        end
        if ui.buy_booster and DCEI.FrameExists(ui.buy_booster.Frame) then
            ui.screen_mask.Hide()
            DCEI.DestroyFrame(ui.buy_booster.Frame)
            ui.buy_booster = nil
        end
        local layout = shop.GetItemLayout(item_data.product_id)
        if layout then
            layout:Hide()
        end
    elseif item_data.product_id == "com.wildarmy.test.iap.resource_bundle.01" then
        local endless_mode = GetBankData("endless_mode")
        endless_mode.resource_bundle = true
        DCEI.TriggerAddTimerEventElapsed(function()
            game_shop.CreateEndlessBundles(endless.page)
        end, 0.5, true, true)
        local skip_fly = menu.in_game
        meta.AddGold(nil, 2000, skip_fly)
        if not endless_mode.event_over or endless_mode.unclaimed_rewards then
            BlockInput()
            endless_mode.gold = endless_mode.gold * 2
            endless_mode.gem = endless_mode.gem * 2
            endless_mode.bang = endless_mode.bang * 2
            endless_mode.punch = endless_mode.punch * 2
            DCEI.TriggerAddTimerEventElapsed(function()
                UnblockInput()
                game_shop.CreateEventIsOverPromotion()
            end, 2, true, true)
        else
            if not menu.in_game then
                local start_ui = endless.page.RewardsChest
                if endless_mode.punch > 0 then
                    meta.AddBooster(start_ui, endless_mode.punch, "booster_punch")
                end
                if endless_mode.bang > 0 then
                    meta.AddBooster(start_ui, endless_mode.bang, "booster_bigbang")
                end
                if endless_mode.gem > 0 then
                    meta.AddGem(start_ui, endless_mode.gem)
                end
                if endless_mode.gold > 0 then
                    meta.AddGold(start_ui, endless_mode.gold)
                end
            end
        end
        CommitBankData("endless_mode")
    elseif item_data.product_id == "com.wildarmy.test.iap.hand_of_midas.01" then
        local endless_mode = GetBankData("endless_mode")
        endless_mode.hand_of_midas = true
        CommitBankData("endless_mode")
        if not menu.in_game then
            local function SendRewards(source, target, sprite, amount)
                ui.FlyTo(source, target, endless.page.Frame, amount, sprite, 60, 1, 0.5, function()
                    local k1, k2 = 1.5, 1
                    local duration = 0.25
                    local ease = "InCubic"
                    DCEI.AnimateFrameScale(
                        target,
                        { x = k1, y = k1, z = k1 },
                        { x = k2, y = k2, z = k2 },
                        duration,
                        ease
                    )
                    DCEI.TriggerAddTimerEventElapsed(function()
                        game_shop.CreateEndlessBundles(endless.page)
                    end, 0.5, true, true)
                end)
            end
            SendRewards(
                endless.page.BundleScroll,
                endless.page.PlayButton,
                "cartoon_icon_token_brown_mergewar",
                endless_mode.token_bank
            )
            endless.midas_pip = true
        else
            if ui.game then
                endless.CollectTokenFromBank(ui.game)
            end
            game_shop.CreateEndlessBundles(endless.page)
        end
    elseif item_data.product_id == "com.wildarmy.test.iap.bundle.live_event01" then
        local event_unit_name = live_event.EVENT_NAME
        local chest = {
            { name = "gem", num = items.gem },
            { name = event_unit_name, num = items.unit_shards },
            { name = "event_key", num = items.event_key },
            chest_unit = "ChestGold",
        }
        live_event.ShowEventUI(false)
        local bank_data = GetBankData("live_event")
        bank_data.bunlde_count = bank_data.bunlde_count or 0
        bank_data.bunlde_count = bank_data.bunlde_count + 1
        CommitBankData("live_event")
        chest.on_close = function()
            live_event.ShowEventUI(true)
            live_event.UpdateChestPage()
        end
        meta.OpenChest(chest)
    elseif item_data.product_id == "com.wildarmy.test.iap.bundle.live_event_pass01" then
        local gold_value = items.gold
        local skip_fly = true
        meta.AddGold(nil, gold_value, skip_fly)
        local message = "<size=200%><sprite name=cartoon_icon_mergewar_coin3> +" .. FormatWithCommas(gold_value)
        ui.FeedbackMessage(message)
        local live_event_protect = GetBankData("live_event_protect")
        live_event_protect.pass = true
        CommitBankData("live_event_protect")
        live_event.UpdateChestPage()
    elseif item_data.type == "talent_stone" then
        local chest = {
            { name = "gold", num = items.gold },
            { name = "talent_stone", num = items.talent_stone },
            chest_unit = "ChestGold",
        }
        chest.on_close = function()
            menu.pages[1]:UpdateTalentStoneItems()
        end
        meta.OpenChest(chest)
    else
        if items then
            -- core.LogDump(items)
            if items._remove_ad then
                local message = DCEI.GetTranslationText(
                    "ui/purchase_success",
                    { x = DCEI.GetTranslationText("ui/remove_ads/title") }
                )
                ui.FeedbackMessage(message)
                local shop_state = GetBankData("shop_state")
                shop_state.remove_ads = true
                CommitBankData("shop_state")
                game_shop.UpdateStoreWatchAdItems()
                local layout = shop.GetItemLayout(item_data.product_id)
                if layout then
                    layout:Hide()
                end
            end

            -- deliver gold
            if items.gold then
                local start_ui = shop.GetItemLayout(product_id) and shop.GetItemLayout(product_id).Frame or ui.root
                meta.AddGold(start_ui, items.gold)
            end

            -- deliver gems
            if items.gems then
                local value = items.gems
                if item_data.type == "gem" then
                    local first_time = CheckFirstTimeBonus(item_data.product_id)
                    if first_time then
                        value = value + value
                        local bank_data = GetBankData("shop_1st_time_bonus")
                        bank_data[product_id] = true
                        CommitBankData("shop_1st_time_bonus")
                        local item_layout = shop.GetItemLayout(item_data.product_id)
                        if item_layout then
                            UpdateStoreGemDisplay(item_layout)
                        end
                    end
                end
                local start_ui = shop.GetItemLayout(product_id) and shop.GetItemLayout(product_id).Frame or ui.root
                meta.AddGem(start_ui, value)
            end

            if items.ads_card then
                local chest = GenerateAdsCardsChest()
                if game_shop.need_rotate_dailyads then
                    chest.on_close = function()
                        menu.pages[1]:StoreWatchAdItemRotateAnimation()
                    end
                end
                meta.OpenChest(chest)
            end

            if items.chest then
                local chest = meta.GenerateRareChest()
                if game_shop.need_rotate_dailyads then
                    chest.on_close = function()
                        menu.pages[1]:StoreWatchAdItemRotateAnimation()
                    end
                end
                meta.OpenChest(chest)
                local bank_tutorial = GetBankData("tutorial_state")
                if not bank_tutorial.campaign_chest then
                    bank_tutorial.campaign_chest = true
                    CommitBankData("tutorial_state")
                    DCEI.Event.LogRegistrationComplete("First Chest")
                end
            end

            if items.bigchest then
                local chest = meta.GenerateEpicChest()
                meta.OpenChest(chest)
            end

            if items.bigestchest then
                local chest = meta.GenerateMysticChest()
                meta.OpenChest(chest)
            end

            if item_data.type == "elixir" then
                for type, value in pairs(items) do
                    if type ~= "type" then
                        meta.BoughtElixir(type)
                    end
                end
            elseif item_data.type == "gem" then
            end
        end
    end

    if item_data.reset_popup then
        local shop_state = GetBankData("shop_state")
        shop_state.daily_gamestart_popup = nil
        CommitBankData("shop_state")
    end

    local purchase_log = GetBankData("purchase_log")
    local bank_calender = GetBankData("calender")
    local usd_price = shop.GetProductPriceUSD(item_data.product_id)
    if usd_price and not currency then
        local day = bank_calender.day
        local entry = { core.Round(usd_price), day }
        table.insert(purchase_log, entry)
        CommitBankData("purchase_log")
    end
    -- log item purchase
    currency = currency or "cash"
    local price_raw = item_data:GetPriceData().usd or 0
    local price = string.format("%.2f", price_raw)
    local total_progress = GetBankData("total_progress")
    LogEvent(LogEvents.BuyItem(item_data.product_id, currency, price, total_progress))
    local money_spent = GetBankData("money_spent") + price
    CommitBankData("money_spent", money_spent)
    LogUserProperty(UserProps.money_spent)
end

function game_shop.CheckPopupOnGameStart()
    local shop_page = menu.pages[1]
    local shop_state = GetBankData("shop_state")
    if not shop_state.daily_gamestart_popup then
        local popup_type = "Open Game"
        -- popup list order, at most 2 popup items
        if meta.CheckLevelClear(1, 6) then
            local product_id = "com.wildarmy.test.iap.bundle.starter01"
            local stock_remaining = shop.GetItemStockAvailable(product_id)
            if stock_remaining > 0 then
                -- DCEI.LogError("Insert: "..product_id)
                table.insert(shop_page.popup_list, { product_id = product_id, type = popup_type })
            end
        end

        if meta.CheckLevelClear(1, 11) then
            local product_id = "com.wildarmy.test.iap.bundle.goldcard"
            local card_data = GetBankData("daily_reward_card")
            if not card_data.gold_card then
                -- DCEI.LogError("Insert: "..product_id)
                table.insert(shop_page.popup_list, { product_id = product_id, type = popup_type })
            end
        end

        if meta.CheckLevelClear(1, 15) then
            local product_id = "com.wildarmy.test.iap.bundle.booster01"
            local stock_remaining = shop.GetItemStockAvailable(product_id)
            if stock_remaining > 0 then
                -- DCEI.LogError("Insert: "..product_id)
                table.insert(shop_page.popup_list, { product_id = product_id, type = popup_type })
            end
        end

        if meta.CheckLevelClear(1, 6) then
            local product_id = "com.wildarmy.test.iap.bundle.starter01"
            local stock_remaining = shop.GetItemStockAvailable(product_id)
            if stock_remaining <= 0 then
                local product_id = "com.wildarmy.test.iap.bundle.elite03"
                local stock_remaining = shop.GetItemStockAvailable(product_id)
                if stock_remaining > 0 then
                    -- DCEI.LogError("Insert: "..product_id)
                    table.insert(shop_page.popup_list, { product_id = product_id, type = popup_type })
                end
            end
        end

        if meta.CheckLevelClear(1, 6) then
            local product_id = "com.wildarmy.test.iap.bundle.shard02"
            local stock_remaining = shop.GetItemStockAvailable(product_id)
            if stock_remaining > 0 then
                -- DCEI.LogError("Insert: "..product_id)
                table.insert(shop_page.popup_list, { product_id = product_id, type = popup_type })
            end
        end

        while #shop_page.popup_list > 2 do
            table.remove(shop_page.popup_list, #shop_page.popup_list)
        end
    end
end

function game_shop.CheckStarterBundlePopup()
    local world = GetBankData("world")
    if world == 1 and menu.selected_level == 7 then
        local shop_state = GetBankData("shop_state")
        if not shop_state.daily_starter_popup then
            shop_state.daily_starter_popup = true
            CommitBankData("shop_state")
            local shop_page = menu.pages[1]
            local product_id = "com.wildarmy.test.iap.bundle.starter01"
            local stock_remaining = shop.GetItemStockAvailable(product_id)
            if stock_remaining > 0 then
                table.insert(shop_page.popup_list, { product_id = product_id, type = "After Stage" })
            end
        end
    end
end

function game_shop.CheckBoosterBundlePopup()
    local world = GetBankData("world")
    if world == 1 and menu.selected_level == 16 then
        local shop_state = GetBankData("shop_state")
        if not shop_state.daily_booster_popup then
            shop_state.daily_booster_popup = true
            CommitBankData("shop_state")
            local shop_page = menu.pages[1]
            local product_id = "com.wildarmy.test.iap.bundle.booster01"
            local stock_remaining = shop.GetItemStockAvailable(product_id)
            if stock_remaining > 0 then
                table.insert(shop_page.popup_list, { product_id = product_id, type = "After Stage" })
            end
        end
    end
end

function game_shop.CheckGoldCardPopup(type, talent_tree)
    local world = GetBankData("world")
    local shop_state = GetBankData("shop_state")
    if
        world == 1
        and (menu.selected_level >= 8 and menu.selected_level <= 11)
        and not shop_state.daily_popup_goldcard
    then
        local shop_page = menu.pages[1]
        local gold = GetBankData("gold")
        local upgrade_talent_gold = talent_tree.GetNextUpgradeGold()
        local product_id = "com.wildarmy.test.iap.bundle.goldcard"
        local card_data = GetBankData("daily_reward_card")
        if gold < upgrade_talent_gold and not card_data.gold_card then
            for index, item in ipairs(shop_page.popup_list) do
                if item.product_id == product_id then
                    table.remove(shop_page.popup_list, index)
                    break
                end
            end
            table.insert(shop_page.popup_list, { product_id = product_id, type = type })
        end
    end
end

function game_shop.CheckShopPopup()
    if meta.run_weekly_actions then
        return
    end
    if GetBankData("dev").auto_start_battle then
        return
    end
    if live_event.ftue_open_chest then
        return
    end
    if meta.CheckTutorialStateEmpty() then
        local shop_page = menu.pages[1]
        if shop_page.popup_list and #shop_page.popup_list > 0 then
            if shop.GetItemStockAvailable(shop_page.popup_list[1].product_id) then
                meta.CloseWatchAdsPrompt()
                meta.PauseAllChestsAnimation()
                local items_list = { shop_page.popup_list[1].product_id }
                shop_page.pop_up:Show(items_list)
                LogEvent(
                    LogEvents.ShopPopup(
                        shop_page.popup_list[1].product_id,
                        shop_page.popup_list[1].type,
                        GetBankData("total_progress")
                    )
                )
                if
                    shop_page.popup_list[1].product_id == "com.wildarmy.test.iap.bundle.goldcard"
                    and shop_page.popup_list[1].type ~= "Open Game"
                then
                    local shop_state = GetBankData("shop_state")
                    shop_state.daily_popup_goldcard = true
                    CommitBankData("shop_state")
                end
                if shop_page.popup_list[1].type == "Open Game" then
                    local shop_state = GetBankData("shop_state")
                    shop_state.daily_gamestart_popup = true
                    CommitBankData("shop_state")
                end
            end
            table.remove(shop_page.popup_list, 1)
        else
            meta.ResumeChestsAnimation()
        end
    end
end

function game_shop.CreateRemoveAdsPromotion(close_callback)
    local layout = gmui.Layout:New({
        parent = ui.root,
        name = "Meta/RemoveAdsPromotion",
    })

    local text1 = DCEI.GetTranslationText("ui/remove_ads/promotion/1", { x = 5 })
    local text2 = DCEI.GetTranslationText("ui/remove_ads/promotion/2", { x = "25x" })
    local text3 = DCEI.GetTranslationText("ui/remove_ads/promotion/3")
    local text4 = DCEI.GetTranslationText("ui/remove_ads/promotion/4")

    DCEI.SetTextFrameText(layout.P1, text1)
    DCEI.SetTextFrameText(layout.P2, text2)
    DCEI.SetTextFrameText(layout.P3, text3)
    DCEI.SetTextFrameText(layout.P4, text4)

    local product_id = "com.wildarmy.test.iap.remove_ad.01"
    -- add "USD" suffix to full price display if localized price exists and isn't USD
    local localized_price = shop.GetProductPriceLocalized(product_id)
    if not localized_price then
        localized_price = "¥" .. shop.GetProductPriceUSD(product_id)
    end
    DCEI.SetTextFrameText(layout.CostLabel, localized_price)

    DCEI.SetOnClickCallback(layout.BackButton, function()
        if close_callback then
            close_callback()
        end
        DCEI.DestroyFrame(layout.Frame)
    end)

    DCEI.SetOnClickCallback(layout.PurchaseButton, function()
        game_shop.AttemptAnyItemPurchase(product_id)
        DCEI.TriggerAddTimerEventElapsed(function()
            if close_callback then
                close_callback()
            end
        end, 1, false, true)

        DCEI.DestroyFrame(layout.Frame)
    end)
end

function game_shop.AttemptStartShopAdsTimer(remaining_time)
    remaining_time = math.min(remaining_time, meta.five_hour)

    if game_shop.shop_ads_timer or remaining_time <= 0 then
        return
    end

    local timer = core.OsTimer:New()
    game_shop.shop_ads_timer = timer
    timer:AddDuration(remaining_time)

    function timer:Tick()
        local time = self:GetTimeRemaining()
        UpdateShopAdsTimer(time)
    end

    function timer:Action()
        UpdateShopAdsTimer(0)
        game_shop.shop_ads_timer = nil
        game_shop.shop_ads_item_ready = true
        game_shop.UpdateStoreWatchAdItems()
        game_shop.CheckShopButtonPip()
    end

    timer:Tick()
end

function game_shop.CreateEventIsOverPromotion()
    if ui.endless_event_over and DCEI.FrameExists(ui.endless_event_over.Frame) then
        ui.endless_event_over:Show()
        ui.endless_event_over:Update()
    else
        local event_over = gmui.Layout:New({
            parent = ui.root,
            name = "Meta/Page5/EventIsOver",
        })
        ui.endless_event_over = event_over
        function event_over:Update()
            local frame_state = DCEI.GetFrameState(event_over.Frame)
            local endless_mode = GetBankData("endless_mode")
            if endless_mode.event_over then
                frame_state.is_over = true
                local text = DCEI.GetTranslationText("meta/page5/eventisover/description")
                DCEI.SetTextFrameText(event_over.Description, text)
            else
                frame_state.is_over = false
            end
            frame_state.can_claim = endless_mode.unclaimed_rewards

            local text = DCEI.GetTranslationText(
                "shop/card/immediate",
                { x = "<sprite name=cartoon_icon_mergewar_coin3><color=yellow>2000</color>" }
            )
            DCEI.SetTextFrameText(event_over.ImmediateGoldText, text)

            local gold = 0
            local gem = 0
            local bang = 0
            local punch = 0
            if endless_mode.gold then
                gold = endless_mode.gold
            end
            if endless_mode.gem then
                gem = endless_mode.gem
            end
            if endless_mode.bang then
                bang = endless_mode.bang
            end
            if endless_mode.punch then
                punch = endless_mode.punch
            end
            if endless_mode.resource_bundle then
                frame_state.has_bundle = true
                punch = math.floor(punch * 0.5)
                gem = math.floor(gem * 0.5)
                gold = math.floor(gold * 0.5)
                bang = math.floor(bang * 0.5)

                DCEI.SetTextFrameText(event_over.GoldMultiplier, gold)
                DCEI.SetTextFrameText(event_over.GemMultiplier, gem)
                DCEI.SetTextFrameText(event_over.BigBangMultiplier, bang)
                DCEI.SetTextFrameText(event_over.EnergyPunchMultiplier, punch)
            else
                frame_state.has_bundle = false
                DCEI.SetTextFrameText(event_over.GoldMultiplier, gold)
                DCEI.SetTextFrameText(event_over.GemMultiplier, gem)
                DCEI.SetTextFrameText(event_over.BigBangMultiplier, bang)
                DCEI.SetTextFrameText(event_over.EnergyPunchMultiplier, punch)
            end

            DCEI.SetOnClickCallback(event_over.ClaimButton, function()
                if endless_mode.unclaimed_rewards then
                    meta.AddBooster(event_over.RewardEnergyPunch, endless_mode.punch, "booster_punch")
                    meta.AddBooster(event_over.RewardBigBang, endless_mode.bang, "booster_bigbang")
                    meta.AddGem(event_over.RewardGem, endless_mode.gem)
                    meta.AddGold(event_over.RewardGold, endless_mode.gold)
                    endless_mode.unclaimed_rewards = false
                    DCEI.SetFrameActive(endless.page.RewardsPip, false)
                    CommitBankData("endless_mode")
                    event_over:Hide()
                else
                    event_over:Hide()
                end
            end)

            local product_id = "com.wildarmy.test.iap.resource_bundle.01"
            DCEI.SetOnClickCallback(event_over.ResourceButton, function()
                event_over:Hide()
                game_shop.AttemptAnyItemPurchase(product_id)
            end)

            -- add "USD" suffix to full price display if localized price exists and isn't USD
            local localized_price = shop.GetProductPriceLocalized(product_id)
            if not localized_price then
                DCEI.SetTextFrameText(event_over.BundleCost, "¥17")
            else
                DCEI.SetTextFrameText(event_over.BundleCost, "¥17")
            end
        end

        event_over:Update()
    end

    return ui.endless_event_over
end

function game_shop.CreateEndlessBundles(page, is_initial)
    local endless_mode = GetBankData("endless_mode")

    if not is_initial then
        if endless_mode.resource_bundle or endless_mode.hand_of_midas then
            if page.virtual_bundle_list then
                DCEI.DestroyFrame(page.BundleContent)
                page.virtual_bundle_list = nil
            end
            DCEI.SetFrameActive(page.RightButton, false)
            DCEI.SetFrameActive(page.LefttButton, false)
            if endless_mode.resource_bundle and endless_mode.hand_of_midas then
                DCEI.DestroyFrame(page.MidasContent)
                DCEI.DestroyFrame(page.ResourceContent)
                DCEI.SetFrameActive(page.EmptyPanel, true)
                return
            else
                if endless_mode.resource_bundle then
                    DCEI.SetFrameActive(page.ResourceScroll, true)
                    local virtual_bundle_list = DCEI.CreateListFrame(page.ResourceContent)
                    DCEI.SetListFrameItemSize(virtual_bundle_list, 725)
                    DCEI.SetListFrameItemCount(virtual_bundle_list, 1)
                    DCEI.SetListFrameItemName(virtual_bundle_list, "Meta/Page5/Bundle")
                    DCEI.SetListFrameItemDataCallback(virtual_bundle_list, function(item, index)
                        local state = DCEI.GetFrameState(item)
                        local MidasButton = DCEI.GetChildFrameById(item, "MidasButton")
                        local MidasCost = DCEI.GetChildFrameById(item, "MidasCost")
                        local TokenBonus = DCEI.GetChildFrameById(item, "TokenBonus")
                        state.is_midas = true
                        if endless_mode.hand_of_midas then
                            state.has_bundle = true
                            DCEI.SetFrameActive(item, false)
                        else
                            state.has_bundle = false
                        end
                        DCEI.SetTextFrameText(TokenBonus, "+" .. endless_mode.token_bank)
                        local product_id = "com.wildarmy.test.iap.hand_of_midas.01"
                        local localized_price = shop.GetProductPriceLocalized(product_id)
                        if not localized_price then
                            DCEI.SetTextFrameText(MidasCost, "¥24")
                        else
                            DCEI.SetTextFrameText(MidasCost, "¥24")
                        end
                        DCEI.SetOnClickCallback(MidasButton, function()
                            local product_id = "com.wildarmy.test.iap.hand_of_midas.01"
                            game_shop.AttemptAnyItemPurchase(product_id)
                            state.has_bundle = true
                        end)
                        local immediate_gold_text = DCEI.GetChildFrameById(item, "ImmediateGoldText")
                        local text = DCEI.GetTranslationText(
                            "shop/card/immediate",
                            { x = "<sprite name=cartoon_icon_mergewar_coin3><color=yellow>2000</color>" }
                        )
                        DCEI.SetTextFrameText(immediate_gold_text, text)
                    end)
                    page.virtual_bundle_list = virtual_bundle_list
                else
                    DCEI.SetFrameActive(page.MidasScroll, true)
                    local virtual_bundle_list = DCEI.CreateListFrame(page.MidasContent)
                    DCEI.SetListFrameItemSize(virtual_bundle_list, 725)
                    DCEI.SetListFrameItemCount(virtual_bundle_list, 1)
                    DCEI.SetListFrameItemName(virtual_bundle_list, "Meta/Page5/Bundle")
                    DCEI.SetListFrameItemDataCallback(virtual_bundle_list, function(item, index)
                        local state = DCEI.GetFrameState(item)
                        local ResourceButton = DCEI.GetChildFrameById(item, "ResourceButton")
                        local ResourceCost = DCEI.GetChildFrameById(item, "ResourceCost")
                        state.is_midas = false
                        if endless_mode.resource_bundle then
                            state.has_bundle = true
                            DCEI.SetFrameActive(item, false)
                        else
                            state.has_bundle = false
                        end
                        local product_id = "com.wildarmy.test.iap.resource_bundle.01"
                        local localized_price = shop.GetProductPriceLocalized(product_id)
                        if not localized_price then
                             DCEI.SetTextFrameText(ResourceCost, "¥17")
                        else
                             DCEI.SetTextFrameText(ResourceCost, "¥17")
                        end
                        DCEI.SetOnClickCallback(ResourceButton, function()
                            local product_id = "com.wildarmy.test.iap.resource_bundle.01"
                            game_shop.AttemptAnyItemPurchase(product_id)
                            state.has_bundle = true
                        end)
                    end)
                    page.virtual_bundle_list = virtual_bundle_list
                end
            end
        end
    else
        local virtual_bundle_list = DCEI.CreateListFrame(page.BundleContent)

        DCEI.SetListFrameItemSize(virtual_bundle_list, 725)
        DCEI.SetListFrameItemCount(virtual_bundle_list, 4)
        DCEI.SetListFrameItemName(virtual_bundle_list, "Meta/Page5/Bundle")
        DCEI.SetListFrameItemDataCallback(virtual_bundle_list, function(item, index)
            local state = DCEI.GetFrameState(item)
            local MidasButton = DCEI.GetChildFrameById(item, "MidasButton")
            local ResourceButton = DCEI.GetChildFrameById(item, "ResourceButton")
            local MidasCost = DCEI.GetChildFrameById(item, "MidasCost")
            local ResourceCost = DCEI.GetChildFrameById(item, "ResourceCost")
            local TokenBonus = DCEI.GetChildFrameById(item, "TokenBonus")
            if index % 2 == 0 then
                state.is_midas = true
                if endless_mode.hand_of_midas then
                    state.has_bundle = true
                    DCEI.SetFrameActive(item, false)
                else
                    state.has_bundle = false
                end
                DCEI.SetTextFrameText(TokenBonus, "+" .. endless_mode.token_bank)
                local product_id = "com.wildarmy.test.iap.hand_of_midas.01"
                local localized_price = shop.GetProductPriceLocalized(product_id)
                if not localized_price then
                  DCEI.SetTextFrameText(MidasCost, "¥24")
                else
                  DCEI.SetTextFrameText(MidasCost, "¥24")
                end
            else
                state.is_midas = false
                if endless_mode.resource_bundle then
                    state.has_bundle = true
                    DCEI.SetFrameActive(item, false)
                else
                    state.has_bundle = false
                end
                local product_id = "com.wildarmy.test.iap.resource_bundle.01"
                local localized_price = shop.GetProductPriceLocalized(product_id)
                if not localized_price then
                    DCEI.SetTextFrameText(ResourceCost, "¥17")
                else
                    DCEI.SetTextFrameText(ResourceCost, "¥17")
                end
                local immediate_gold_text = DCEI.GetChildFrameById(item, "ImmediateGoldText")
                local text = DCEI.GetTranslationText(
                    "shop/card/immediate",
                    { x = "<sprite name=cartoon_icon_mergewar_coin3><color=yellow>2000</color>" }
                )
                DCEI.SetTextFrameText(immediate_gold_text, text)
            end
            DCEI.SetOnClickCallback(MidasButton, function()
                local product_id = "com.wildarmy.test.iap.hand_of_midas.01"
                game_shop.AttemptAnyItemPurchase(product_id)
                state.has_bundle = true
            end)
            DCEI.SetOnClickCallback(ResourceButton, function()
                local product_id = "com.wildarmy.test.iap.resource_bundle.01"
                game_shop.AttemptAnyItemPurchase(product_id)
                state.has_bundle = true
            end)
        end)

        local function ArrowButtons(right, duration, ease)
            local position = DCEI.GetScrollFrameScrollPosition(page.BundleScroll)
            local target
            if right then
                if position >= 1449 then
                    position = 0
                end
                local int, remainder = math.modf(position / 725)
                if remainder > 0.999 and remainder < 1.001 then
                    remainder = 0
                end
                target = (725 - (remainder * 725)) + position
            else
                if position <= 1 then
                    position = 1450
                end
                local int, remainder = math.modf(position / 725)
                if remainder > -0.001 and remainder < 0.001 then
                    remainder = 1
                end
                target = position - (remainder * 725)
            end
            local k1, k2 = position, target
            DCEI.AnimateFrameScrollPosition(page.BundleScroll, k1, k2, duration, ease)
        end
        page.virtual_bundle_list = virtual_bundle_list
        local time = 0
        local timer = core.OsTimer:New()
        local arrows = false
        function timer:Condition()
            return not DCEI.FrameExists(page.BundleContent)
        end

        function timer:Tick()
            time = time + 1
            if time >= 4 then
                ArrowButtons(true, 1, "OutQuint")
                time = 0
            end
        end

        DCEI.SetOnClickCallback(page.RightButton, function()
            ArrowButtons(true, 0.35, "Linear")
            time = 0
        end)
        DCEI.SetOnClickCallback(page.LefttButton, function()
            ArrowButtons(false, 0.35, "Linear")
            time = 0
        end)
    end
end

function game_shop.CreateBankPopup()
    local endless_mode = GetBankData("endless_mode")
    local bank_popup = gmui.Layout:New({
        parent = ui.root,
        name = "Gameplay/Endless/BankPopup",
    })
    local product_id = "com.wildarmy.test.iap.hand_of_midas.01"
    local localized_price = shop.GetProductPriceLocalized(product_id)
    DCEI.SetTextFrameText(bank_popup.TokenBonus, "+" .. endless_mode.token_bank)
    if not localized_price then
        DCEI.SetTextFrameText(bank_popup.MidasCost, "¥24")
        DCEI.SetTextFrameText(bank_popup.BuyText, "¥24")
    else
        DCEI.SetTextFrameText(bank_popup.MidasCost, "¥24")
        DCEI.SetTextFrameText(bank_popup.BuyText, "¥24")
    end
    DCEI.SetOnClickCallback(bank_popup.BuyButton, function()
        game_shop.AttemptAnyItemPurchase(product_id)
        DCEI.DestroyFrame(bank_popup.Frame)
    end)
    DCEI.SetOnClickCallback(bank_popup.CloseButton, function()
        DCEI.DestroyFrame(bank_popup.Frame)
    end)
end

function game_shop.CheckShopButtonPip()
    local has_keys = GetBankData("gear_key_common") > 0 or GetBankData("gear_key_magic") > 0
    if meta.CheckLevelClear(1, 4) then
        -- check ads timer ready
        if game_shop.shop_ads_item_ready or has_keys then
            menu.navi_buttons[1].pip:Show()
        else
            menu.navi_buttons[1].pip:Hide()
        end
    end
end

function game_shop.GetMods(_game, _meta, _menu, _endless, _live_event)
    game = _game
    meta = _meta
    menu = _menu
    endless = _endless
    live_event = _live_event
end

return game_shop
